# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 19th November 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis

##########################################
### Illustration: Exponential Data
##########################################
##########################################


##########################################
### Data
##########################################
n <- 1
ybar <- 1


# Likelihood
l <- function(lambda) {
  n * (log(lambda) - lambda * ybar)
}
# Likelihood root
r <- function(lambda) {
  sign(1 - ybar*lambda) * sqrt(2*(l(ybar) - l(lambda)))
}
# Wald pivot
t <- function(lambda) {
  sqrt(n) * (1 - lambda * ybar)
}
# Score pivot
s <- function(lambda) {
  t(lambda)/(lambda*ybar)
}

##########################################
### Log-likelihood and 95% approximate
### confidence interval based on r(lambda)
##########################################
plot(x <- seq(0, 6, length = 1000), l(x), type = "l",
     xlab = expression(lambda), ylab = "Log likelihood", lwd = 2)
abline(l(ybar) - 0.5*qchisq(0.95, 1), 0, col = "green")
w.lower95 <- optimize(function(x) (r(x) - qnorm(0.975))^2,
                      interval = c(0, 1/ybar))$minimum
w.upper95 <- optimize(function(x) (r(x) - qnorm(0.025))^2,
                      interval = c(1/ybar, 1/ybar + 5))$minimum
abline(v = w.lower95, col = "green")
abline(v = w.upper95, col = "green")


##########################################
### Pivots
##########################################
# Exact pivot
plot(x, expiv <- qnorm(1 - pgamma(x * n * ybar, rate = 1, shape = n)), type = "l",
     xlab = expression(lambda), ylab = "Pivot", lwd = 2)
# Likelihood root
points(x, rs <- r(x), type = "l")
# Score pivot
points(x, ss <- s(x), type = "l", lty = 2)
# Wald pivot
points(x, ts <- t(x), type = "l", lty = 3)
abline(h = qnorm(0.025), col = "grey"); abline(h = 1.96, col = "grey")
legend("topright", legend = c("Exact pivot", "Likelihood root",
                     "Score pivot", "Wald pivot"),
       lty = c(1, 1, 2, 3), lwd = c(2, 1, 1, 1))


##########################################
### Significance functions
##########################################
# Exact pivot
plot(x, 1 - pgamma(x * n * ybar, rate = 1, shape = n), type = "l",
     xlab = expression(lambda), ylab = "Pivot", lwd = 2)
# Likelihood root
points(x, pnorm(rs), type = "l")
# Score pivot
points(x, pnorm(ss), type = "l", lty = 2)
# Wald pivot
points(x, pnorm(ts), type = "l", lty = 3)
abline(h = 0.025, col = "grey"); abline(h = 0.975, col = "grey")
legend("topright", legend = c("Exact pivot", "Likelihood root",
                     "Score pivot", "Wald pivot"),
       lty = c(1, 1, 2, 3), lwd = c(2, 1, 1, 1), cex = 0.6)


##########################################
#### Confidence intervals as function of n
##########################################
cis <- function(n, level = 0.95) {
  a <- 1-level
  z <- qnorm(1-a/2)
  r1 <- function(lambda) {
    sign(1 - ybar*lambda)*sqrt(2*n*(ybar*lambda - log(ybar*lambda) - 1))
  }
  exact <- c(qgamma(a/2, shape = n, scale = 1),
             qgamma(1-a/2, , shape = n, scale = 1))/(n*ybar)
  wald.based <- c(1/ybar - z/(sqrt(n)*ybar), 1/ybar + z/(sqrt(n)*ybar))
  score.based <- c(sqrt(n)/(ybar*(z + sqrt(n))), sqrt(n)/(ybar*(-z + sqrt(n))))
  w.lower <- optimize(function(x) (r1(x) - z)^2,
                      interval = c(0, 1/ybar))$minimum
  w.upper <- optimize(function(x) (r1(x) + z)^2,
                        interval = c(1/ybar, 1/ybar + 5))$minimum
  likroot.based <- c(w.lower, w.upper)
  res <- matrix(0, 4, 2)
  dimnames(res) <-  list(c("Exact pivot", "Wald pivot",
                           "Score pivot", "Likelihood root"),
                           c("Lower", "Upper"))
  res[1, ] <- exact; res[2, ] <- wald.based;
  res[3, ] <- score.based ; res[4, ] <- likroot.based
  res
}


##########################################
### Pivots as n increases
##########################################
pivots <- function(n) {
  l <- function(lambda) {
    n * (log(lambda) - lambda * ybar)
  }
  r <- function(lambda) {
    sign(1 - ybar*lambda) * sqrt(2*(l(ybar) - l(lambda)))
  }
  t <- function(lambda) {
    sqrt(n) * (1 - lambda * ybar)
  }
  s <- function(lambda) {
    t(lambda)/(lambda*ybar)
  }
  x <- seq(0, 6, length = 500)
  plot(x, expiv <- qnorm(1 - pgamma(x * n * ybar, rate = 1, shape = n)),
       type = "l", xlab = expression(lambda), ylab = "Pivot", ylim = c(-4, 4),
       lwd = 2)
  title(paste("n = ", n))
  points(x, rs <- r(x), type = "l")
  points(x, ss <- s(x), type = "l", lty = 2)
  points(x, ts <- t(x), type = "l", lty = 3)
  abline(h = - 1.96, col = "grey"); abline(h = 1.96, col = "grey")
  legend("topright", legend = c("Exact pivot", "Likleihood root",
                 "Score pivot", "Wald pivot"),
       lty = c(1, 1, 2, 3), lwd = c(2, 1, 1, 1), cex = 0.6)
}


##########################################
### Plot pivots
##########################################
pivots(1)


for (nn in 2:100) {
  Sys.sleep(0.3)
  pivots(nn)
}

